'use strict';

const path = require('path');

const Koa = require('koa');
const bodyparser = require('koa-bodyparser');
const serve = require('koa-static');
const compress = require('koa-compress');
const views = require('koa-views');
const websockify = require('koa-websocket');

const { createCloneForAllUsers } = require('./routes/sky-id/sky-id/users-clones');
const delay = require('./middleware/delay');
const cors = require('./middleware/cors');
const errorIntercepter = require('./middleware/error-intercepter');
const authIntercepter = require('./middleware/auth-intercepter');
const ovpIntercepter = require('./middleware/ovp-intercepter');
const store = require('./utils/store');

const app = websockify(new Koa());

app.proxy = true;

// Logging
if (process.env.NODE_ENV !== 'production') {
  const logger = require('koa-logger');

  app.use(logger());
}

// Set no-cache headers
app.use(async (ctx, next) => {
  await next();
  ctx.set('Cache-Control', 'no-cache, max-age=0');
});

// CORS
app.use(cors());

// Set cena content range
app.use((ctx, next) => {
  if (ctx.path === '/mp4/jcena.mp4') {
    ctx.set('Content-Range', 'bytes 0-3657614/3657615');
    ctx.status = 206;
    return next();
  } else {
    return next();
  }
});

// Serve static files
app.use(serve(path.join(__dirname, 'public')));

app.use((ctx, next) => {
  const match = ctx.path.match(/images\/(.*)\.png\/\d+x\d+/);
  if (match) {
    ctx.redirect(`/images/${match[1]}.png`);
  } else {
    return next();
  }
});

// Compress
app.use(compress());

// Delay
if (process.env.MOCK_PROXY_DELAY) {
  app.use(delay(process.env.MOCK_PROXY_DELAY));
}

// Views
app.use(
  views(`${__dirname}/views`, {
    extension: 'pug',
    map: {
      pug: 'pug',
    },
  })
);

// Parse body
app.use(
  bodyparser({
    enableTypes: ['json', 'form', 'text'],
    extendTypes: {
      json: [
        'application/json',
        'application/vnd.account-manager.v1+json',
        'application/vnd.paymentaccount.v1+json',
        'application/vnd.enablepin.v1+json',
        'application/vnd.disablepin.v1+json',
        'application/vnd.pinrequired.v1+json',
        'application/vnd.resetpin.v1+json',
        'application/vnd.playlive.v1+json',
        'application/vnd.playvod.v1+json',
        'application/vnd.trailer-vod.v1+json',
        'application/vnd.tokens.v1+json',
        'application/vnd.charges.v1+json',
        'application/vnd.purchase.v1+json',
        'application/vnd.entitlements.v1+json',
        'application/vnd.getproducts.v1+json',
        'application/vnd.vouchers.v1+json',
        'application/vnd.latestsubscriptions.v1+json',
        'application/vnd.cancelrenewal.v1+json',
        'application/vnd.reinstate.v1+json',
        'application/vnd.persona.v1+json',
        'application/vnd.bridge.v1+json',
      ],
    },
  })
);

// Pseudo session store
app.use(store());
app.ws.use(store());

// Error Intercepter
app.use(
  errorIntercepter({
    errSignin: '/signin',
    errSignup: '/signup',
  })
);

// OVP Intercepter
const ovpPathsToIntercept = [
  '/bookmarking/bookmarks',
  '/commerce/account-manager/account',
  '/commerce/account-manager/account/devices',
  '/commerce/account-manager/account/parental-control',
  '/commerce/account-manager/devices',
  '/commerce/bridge/availableproducts',
  '/commerce/bridge/charges',
  '/commerce/bridge/order',
  '/commerce/bridge/user/paymentaccounts',
  '/commerce/bridge/user/paymentpin/pinrequired',
  '/commerce/bridge/voucher/',
  '/commerce/payments-manager/cancelrenewal',
  '/commerce/payments-manager/entitlements',
  '/commerce/payments-manager/orderhistory',
  '/commerce/payments-manager/reinstatesubscription',
  '/commerce/payments-manager/subscriptions/latest',
  '/commerce/payments-manager/user/paymentaccount',
  '/commerce/payments-manager/user/paymentpin/disable',
  '/commerce/payments-manager/user/paymentpin/enable',
  '/commerce/payments-manager/user/paymentpin/reset',
  '/video/playouts/live',
  '/video/playouts/vod',
];
app.use(ovpIntercepter(ovpPathsToIntercept));

// Auth Intercepter
const authPathsToIntercept = ['/auth/tokens', '/auth/users/me'];
app.use(authIntercepter(authPathsToIntercept));

// Load routes
require('./routes')(app);

// this number has to be one more than the number of cucumber processes configured to run in parallel
for (let i = 0; i < 8; i++) {
  createCloneForAllUsers(i);
}

module.exports = app;
