const get = require('lodash/get');

const userIdentifier = require('../../utils/user-identifier');
const { isScenario } = require('../../utils/scenario');

const accountDetailsFixture = require('./commerce/account-manager/account-details');
const accountDetailsOneDeliveryAddressFixture = require('./commerce/account-manager/account-details-one-delivery-address');
const accountDetailsDifferentDeliveryAddressFixture = require('./commerce/account-manager/account-details-different-delivery');
const accountDetailsNoDeliveryAddressFixture = require('./commerce/account-manager/account-details-no-delivery-address');
const accountDetailsNoAddressesFixture = require('./commerce/account-manager/account-details-no-addresses');
const accountDetailsNotFoundFixture = require('./commerce/account-manager/account-details-not-found');
const accountDetailsInvalidFieldFixture = require('./commerce/account-manager/account-details-invalid-field');
const accountDetailsMissingFieldFixture = require('./commerce/account-manager/account-details-missing-field');
const parentalControlOff = require('./commerce/account-manager/parental-control/turned-off');
const parentalControlOn = require('./commerce/account-manager/parental-control/turned-on');
const parentalControlNeverSet = require('./commerce/account-manager/parental-control/never-set');
const cancellationOffer = require('./commerce/account-manager/cancellation-offer');
const noCancellationOfferAvailable = require('./commerce/account-manager/no-cancellation-offer-available');
const notEligibleForCancellationOffer = require('./commerce/account-manager/not-eligible-for-cancellation-offer');

module.exports.getAccountDetails = function(ctx) {
  const userToken = ctx.request.get('X-SkyOTT-UserToken');
  const user = userIdentifier.getDataForUserOrDefault(ctx);
  const storeData = ctx.persistentStore.get('accountDetails');

  if (!storeData && (user.noAccountDetails || isScenario(ctx, 'account_not_found'))) {
    ctx.body = accountDetailsNotFoundFixture;
    return (ctx.status = 404);
  }

  if (isScenario(ctx, 'get_account_details_500')) {
    ctx.body = accountDetailsNotFoundFixture;
    return (ctx.status = 500);
  }

  if (storeData) {
    ctx.body = storeData;
  } else if (user.accountDetailsOneDeliveryAddress) {
    ctx.body = accountDetailsOneDeliveryAddressFixture;
  } else if (user.noAddressesOnAccount) {
    ctx.body = accountDetailsNoAddressesFixture;
  } else if (user.noDeliveryAddressOnAccount) {
    ctx.body = accountDetailsNoDeliveryAddressFixture;
  } else if (user.differentDeliveryAddressOnAccount) {
    ctx.body = accountDetailsDifferentDeliveryAddressFixture;
  } else if (user.invalidAddressFieldOnAccount) {
    ctx.body = accountDetailsInvalidFieldFixture;
  } else if (user.missingAddressFieldOnAccount) {
    ctx.body = accountDetailsMissingFieldFixture;
  } else {
    ctx.body = accountDetailsFixture;
  }

  ctx.status = 200;
  ctx.set('X-SkyOTT-UserToken', userToken);
  ctx.type = 'application/vnd.account-manager.v1+json';
};

module.exports.createOrUpdateAccountDetails = function(ctx) {
  const town = get(ctx.request.body.addresses, ['billing', 0, 'town']);

  ctx.set('X-SkyOTT-UserToken', ctx.request.get('X-SkyOTT-UserToken'));
  ctx.body = null;
  ctx.type = 'application/vnd.account-manager.v1+json';

  if (isScenario(ctx, 'update_account_addresses_error')) {
    ctx.status = 500;
  } else if (town === 'Error') {
    ctx.status = 500;
  } else {
    ctx.persistentStore.set('accountDetails', ctx.request.body);
    ctx.status = 204;
  }
};

module.exports.getParentalControl = function(ctx) {
  if (isScenario(ctx, 'get_parental_control_500')) {
    ctx.status = 500;
    return;
  }

  let data = parentalControlOn;
  const storeData = ctx.persistentStore.get('parentalControl');
  const user = userIdentifier.getDataForUserOrDefault(ctx);

  if (storeData || (user.parentalPin && user.parentalPin.pin)) {
    const userPinData = storeData || {
      rating: '18',
      lastModifiedDate: new Date().toISOString(),
    };
    data = Object.assign({}, parentalControlOn, userPinData);
  } else if (!user.parentalPin || user.parentalPin.parentalPinNeverSet === true) {
    data = parentalControlNeverSet;
  } else if (user.parentalPin.parentalPinRequired === false) {
    data = parentalControlOff;
  }

  ctx.status = 200;
  ctx.set('X-SkyOTT-UserToken', ctx.request.get('X-SkyOTT-UserToken'));
  ctx.body = data;
  ctx.type = 'application/vnd.account-manager.v1+json';
};

module.exports.putParentalControl = function(ctx) {
  if (isScenario(ctx, 'put_parental_control_500')) {
    ctx.status = 500;
    return;
  }
  ctx.persistentStore.set('parentalControl', {
    rating: ctx.request.body.rating,
    lastModifiedDate: new Date().toISOString(),
  });
  ctx.status = 204;
  ctx.set('X-SkyOTT-UserToken', ctx.request.get('X-SkyOTT-UserToken'));
  ctx.body = null;
};

module.exports.resetParentalControlFixtures = function(ctx) {
  ctx.persistentStore.delete('parentalControl');
  ctx.body = `Fixtures for parental control have been reset for user: ${ctx.request.query.userToken}`;
  ctx.status = 200;
};

module.exports.resetAccountDetailsFixtures = function(ctx) {
  ctx.persistentStore.delete('accountDetails');
  ctx.body = `Fixtures for account details have been reset for user: ${ctx.request.query.userToken}`;
  ctx.status = 200;
};

module.exports.getCancellationOffer = function(ctx) {
  const user = userIdentifier.getDataForUserOrDefault(ctx);

  if (isScenario(ctx, 'no_cancellation_offer')) {
    ctx.body = noCancellationOfferAvailable;
    ctx.status = 404;
  } else if (isScenario(ctx, 'get_cancellation_offer_500')) {
    ctx.status = 500;
  } else if (user.ineligibleForCancellationOffer) {
    ctx.body = notEligibleForCancellationOffer;
    ctx.status = 200;
  } else {
    ctx.body = cancellationOffer;
    ctx.status = 200;
  }
  ctx.type = 'application/vnd.account-manager.v1+json';
};
