const userIdentifier = require('../../utils/user-identifier');
const { isScenario } = require('../../utils/scenario');

const paymentPinMissingDataFixture = require('./commerce/payment-pin/payment-pin-missing-mandatory-data');
const paymentPinIncorrectPinFixture = require('./commerce/payment-pin/payment-pin-incorrect-pin');
const paymentPinNotFoundPinFixture = require('./commerce/payment-pin/payment-pin-not-found');
const pinServiceDown = require('./commerce/payment-pin/pin-service-down');

module.exports.pinRequired = function(ctx) {
  const user = userIdentifier.getDataForUserOrDefault(ctx);

  const storeData = ctx.persistentStore.get('paymentPin');
  const userToken = ctx.request.get('X-SkyOTT-UserToken');
  ctx.set('X-SkyOTT-UserToken', userToken);

  const paymentPinNotFound = !user.paymentPin || user.paymentPin.errorCode === 'COM_00005';

  let data;

  if (storeData) {
    data = Object.assign({}, storeData);
  } else {
    data = user.paymentPin || {};
  }

  if (data.pin || data.pinRequired) {
    data = { pinRequired: data.pinRequired };
  }

  if (paymentPinNotFound && !storeData) {
    ctx.body = paymentPinNotFoundPinFixture;
    ctx.status = 404;
  } else {
    ctx.body = data;
    ctx.status = 200;
  }

  if (isScenario(ctx, 'pin_required_service_down')) {
    ctx.status = 500;
    ctx.body = pinServiceDown;
  }
  ctx.set('Content-Type', 'application/vnd.pinrequired.v1+json');
};

module.exports.reset = function(ctx) {
  ctx.set('Content-Type', 'application/vnd.resetpin.v1+json');
  if (isScenario(ctx, 'reset_pin_service_down')) {
    ctx.status = 500;
    ctx.body = pinServiceDown;
  } else if (ctx.request.body.pin) {
    ctx.status = 204;
    ctx.set('X-SkyOTT-UserToken', ctx.request.get('X-SkyOTT-UserToken'));
  } else {
    ctx.status = 400;
    ctx.body = paymentPinMissingDataFixture;
  }
};

module.exports.enable = function(ctx) {
  ctx.set('Content-Type', 'application/vnd.enablepin.v1+json');
  const userToken = ctx.request.get('X-SkyOTT-UserToken');
  const user = userIdentifier.getDataForUserOrDefault(ctx);
  ctx.set('X-SkyOTT-UserToken', userToken);
  if (isScenario(ctx, 'enable_pin_service_down')) {
    ctx.status = 500;
    ctx.body = pinServiceDown;
  } else if (user.badPaymentPin === true) {
    ctx.status = 400;
    ctx.body = paymentPinIncorrectPinFixture;
  } else if (ctx.request.body.pin) {
    ctx.status = 204;
    ctx.persistentStore.set('paymentPin', {
      pinRequired: true,
    });
  } else {
    ctx.status = 400;
    ctx.body = paymentPinMissingDataFixture;
  }
};

module.exports.disable = function(ctx) {
  ctx.set('Content-Type', 'application/vnd.disablepin.v1+json');
  if (isScenario(ctx, 'disable_pin_service_down')) {
    ctx.status = 500;
    ctx.body = pinServiceDown;
  } else if (ctx.request.body.pin) {
    ctx.status = 204;
    ctx.set('X-SkyOTT-UserToken', ctx.request.get('X-SkyOTT-UserToken'));
    ctx.persistentStore.set('paymentPin', {
      pinRequired: false,
    });
  } else {
    ctx.status = 400;
    ctx.body = paymentPinMissingDataFixture;
  }
};

module.exports.resetFixtures = function(ctx) {
  ctx.persistentStore.delete('paymentPin');
  ctx.body = `Fixtures for payment pin have been reset for user: ${ctx.request.query.userToken}`;
  ctx.status = 200;
};
