const userIdentifier = require('../../utils/user-identifier');
const { isScenario } = require('../../utils/scenario');
const getEntitlementsForUser = require('../../routes/commerce/entitlements').getEntitlementsForUser;
const getSubscriptionsForUser = require('../../routes/commerce/subscriptions').getSubscriptionsForUser;

const pinRequired = require('./playouts/pin-required');
const pinInvalid = require('./playouts/pin-invalid');
const pinServiceDown = require('./playouts/pin-service-down');
const notEntitled = require('./playouts/not-entitled');
const queuedPass = require('./playouts/queued-pass');
const contentForPasses = require('./content-for-passes').contentForPasses;
const contentErrors = require('./content-errors').contentErrors;
const deviceErrors = require('./device-limit-errors').deviceErrors;

function getPlayableAssetsForUser(ctx) {
  const entitlements = getEntitlementsForUser(ctx).entitlements || [];
  const subscriptions = getSubscriptionsForUser(ctx).subscriptions || [];

  const entitlementProductIds = entitlements.map(ent => ent.productId);
  const subscriptionProductIds = subscriptions.map(sub => sub.product.id);
  const ownedProducts = entitlementProductIds.concat(subscriptionProductIds);

  const playableAssets = ownedProducts.reduce((acc, ownedProduct) => {
    const playableAssetsForOwnedProduct = contentForPasses[ownedProduct];
    if (playableAssetsForOwnedProduct) {
      playableAssetsForOwnedProduct.forEach(playableAsset => acc.add(playableAsset));
    }

    return acc;
  }, new Set());

  return playableAssets;
}

function userCanPlayAsset(ctx, assetToPlay) {
  const playableAssets = getPlayableAssetsForUser(ctx);
  return playableAssets.has(assetToPlay) || playableAssets.has(Number(assetToPlay));
}

function hasPinOverride(ctx) {
  return ctx.request.get('x-skyott-pinOverride') === 'true';
}

function getPlayoutResponse(ctx, successFixture) {
  const userToken = ctx.request.get('X-SkyOTT-UserToken');
  ctx.set('X-SkyOTT-UserToken', userToken);
  const user = userIdentifier.getDataForUserOrDefault(ctx);

  const isPinCorrect = ctx.request.body.parentalControlPin === user.parentalPin.parentalPin;
  ctx.set('Content-Type', 'application/json');

  if (ctx.request.body.pass) {
    ctx.persistentStore.set('playoutPass', ctx.request.body.pass);
  }

  let hasPurchasedQueuedPass;
  const purchasedProductIds = ctx.persistentStore.get('purchased');
  if (purchasedProductIds) {
    hasPurchasedQueuedPass = purchasedProductIds.some(id => {
      try {
        const purchasedProduct = require(`../commerce/commerce/entitlements/${id}.json`);
        return purchasedProduct.state === 'QUEUED';
      } catch (e) {
        return false;
      }
    });
  }

  const canPlayAsset = userCanPlayAsset(ctx, ctx.request.body.contentId || ctx.request.body.serviceKey);

  if (!canPlayAsset) {
    ctx.status = 403;
    ctx.body = notEntitled;
    return;
  }

  if (
    (hasPurchasedQueuedPass && !ctx.persistentStore.get('playoutPass')) ||
    (user.hasQueuedPass && !ctx.request.body.pass && !ctx.persistentStore.get('playoutPass'))
  ) {
    ctx.status = 403;
    ctx.body = queuedPass;
    return;
  }

  if (ctx.request.body.contentId in contentErrors) {
    const contentError = contentErrors[ctx.request.body.contentId];
    ctx.status = contentError.status;
    ctx.body = contentError;
    return;
  }

  if (ctx.request.body.contentId in deviceErrors) {
    const deviceError = deviceErrors[ctx.request.body.contentId];
    ctx.status = deviceError.status;
    ctx.body = deviceError;
    return;
  }

  if (!hasPinOverride(ctx)) {
    if (user.parentalPin.parentalPinRequired && !ctx.request.body.parentalControlPin) {
      ctx.status = 403;
      ctx.body = pinRequired;
      return;
    }

    if (isScenario(ctx, 'pin_service_down')) {
      ctx.status = 500;
      ctx.body = pinServiceDown;
      return;
    }

    if (user.parentalPin.parentalPinRequired && !isPinCorrect) {
      ctx.status = 403;
      ctx.body = pinInvalid;
      return;
    }
  }

  ctx.status = 200;
  ctx.body = successFixture;
}

module.exports = getPlayoutResponse;
