'use strict';

const fs = require('fs');
const path = require('path');

const lodashFind = require('lodash/find');
const lodashFindIndex = require('lodash/findIndex');

const { isScenario } = require('../../utils/scenario');
const { replaceLocalhostWithOrigin } = require('../../utils/replace-localhost-with-origin');

const getModifyUserProfileFixture = require('./sky-id/get-modify-user-profile');
const signUpFixture = require('./sky-id/sign-up');
const signUpCaptchaFixture = require('./sky-id/sign-up-captcha');
const signUpCaptchaErrorFixture = require('./sky-id/sign-up-captcha-invalid');
const signUpFixtureErrors = require('./sky-id/sign-up-errors');
const signUpCaptchaDataFixture = require('./sky-id/sign-up-captcha-data');
const signInFixture = require('./sky-id/sign-in');
const signUpInSuccessFixture = require('./sky-id/sign-up-in-success');
const signInIncorrectCredentialsResponseBody = require('./sky-id/sign-in-incorrect-credentials');
const signInCaptchaResponseBody = require('./sky-id/sign-in-captcha');
const signInCaptchaDataResponseBody = require('./sky-id/sign-in-captcha-data');
const signInWrongCaptchaResponseBody = require('./sky-id/sign-in-wrong-captcha');
const users = require('./sky-id/users');
const resetPasswordForm = require('./sky-id/reset-password-form');
const resetPasswordResponse = require('./sky-id/reset-password-response');
const newPasswordForm = require('./sky-id/new-password-form');
const newPasswordFormExpired = require('./sky-id/new-password-form-expired');
const newPasswordResponse = require('./sky-id/new-password-response');
const newPasswordResponseExpired = require('./sky-id/new-password-response-expired');
const newPasswordResponseAlreadyUsed = require('./sky-id/new-password-response-alreadyused');
const { createCloneForAllUsers, removeCloneForAllUsers } = require('./sky-id/users-clones');

function generateTokenResponse(username) {
  const response = signUpInSuccessFixture;
  if (users[username]) {
    const user = users[username];
    response.properties = {
      eventType: 'success',
      href: `nowtv://auth?access_token=${user.skyIdToken}&token_type=token`,
      data: {
        session: {
          secureToken: user.skyIdToken,
        },
      },
    };
  }

  return response;
}

function setAuthCookies(res, username) {
  const user = users[username];

  if (user) {
    let cookiesToDrop = ['skyCEsidismesso01', 'skyCEsidmesso01', 'skySSO'];

    if (username.includes('keepMeSignedInOnlyUser')) {
      cookiesToDrop = ['skyCEsidexsso01'];
    }

    cookiesToDrop.forEach(cookie => {
      res.cookies.set(cookie, user.skyIdToken, { httpOnly: false, path: '/' });
    });
  }
}

function setCookiesAndGetSignInSuccessResponse(res, username) {
  setAuthCookies(res, username);
  return generateTokenResponse(username);
}

function isCorrectSkyIdHeader(header) {
  return typeof header['x-skyott-territory'] !== 'undefined' && typeof header['x-skyott-provider'] !== 'undefined';
}

function isCorrectSkyIdCredentials(username, password) {
  return typeof users[username] !== 'undefined' && users[username].password === password;
}

function isSignInCaptchaUser(username) {
  return typeof users[username] !== 'undefined' && Boolean(users[username].captchaSolution);
}

function isCaptchaSolutionCorrect(captcha, username) {
  return (
    typeof users[username] !== 'undefined' &&
    Boolean(users[username].captchaSolution) &&
    users[username].captchaSolution === captcha
  );
}

function isCorrectContentType(header) {
  return header['content-type'].startsWith('application/x-www-form-urlencoded');
}

function getRequiredFields() {
  return Object.keys(signUpFixtureErrors.required);
}

function getCreateProfileValidationErrors(formFieldsWithErrors) {
  return formFieldsWithErrors.map(targetField => signUpFixtureErrors.required[targetField]);
}

function validateCreateProfile(formFields) {
  const requiredFields = getRequiredFields();
  if (formFields) {
    const fieldsWithErrors = requiredFields.filter(requiredField => !formFields[requiredField]);
    return getCreateProfileValidationErrors(fieldsWithErrors);
  } else {
    return getCreateProfileValidationErrors(requiredFields);
  }
}

function setPreCaptchaCredentialsCookies(res, username, password) {
  if (isCorrectSkyIdCredentials(username, password)) {
    res.cookies.set('preCaptchaCredentialsCorrect', 'true', { httpOnly: false });
  } else {
    res.cookies.set('preCaptchaCredentialsCorrect', 'false', { httpOnly: false });
  }
  res.cookies.set('preCaptchaUserName', username, { httpOnly: false });
}

module.exports.getSignupForm = function(ctx) {
  let fixture;
  if (isScenario(ctx, 'sign_up_captcha_required')) {
    fixture = replaceLocalhostWithOrigin(signUpCaptchaFixture, ctx);
  } else {
    fixture = replaceLocalhostWithOrigin(signUpFixture, ctx);
  }
  ctx.body = fixture;
};

module.exports.getSigninForm = function(ctx) {
  const header = ctx.request.header;
  const referral = ctx.request.header.referer || '';

  if (!isCorrectSkyIdHeader(header)) {
    return (ctx.status = 400);
  } else if (referral.includes('SIGN_IN_ERROR')) {
    return (ctx.status = 500);
  } else if (referral.includes('SIGN_IN_MALFORM')) {
    return (ctx.body = { class: ['signin'] });
  }

  ctx.body = replaceLocalhostWithOrigin(signInFixture, ctx);
};

module.exports.getProfile = function(ctx) {
  const userSkyIdToken = ctx.cookies.get('skyCEsidmesso01') || ctx.cookies.get('skyCEsidismesso01');

  if (
    !userSkyIdToken ||
    ctx.persistentStore.get('invalidate-login') ||
    isScenario(ctx, 'get_profile_expired_messo_token')
  ) {
    ctx.persistentStore.delete('invalidate-login');
    ctx.status = 401;
    ctx.body = {
      class: ['redirect'],
      properties: {
        targetApi: 'signin',
        type: 'redirectApi',
        eventType: 'error',
        href:
          'https://uiapi.id.func.prod.nowtv.com/signin/service/international?continuationUrl=https%3A%2F%2Fuiapi.id.func.prod.nowtv.com%2Fmodifyprofile%2Fservice%2Finternational%3Fapi_id%3Dmodifyprofile',
      },
    };
  } else {
    const user = Object.values(users).find(u => u.skyIdToken === userSkyIdToken && u.getProfile) || users.valid_user;
    ctx.body = user.getProfile;
  }
};

module.exports.getModifyUserProfile = function(ctx) {
  const messoToken = ctx.cookies.get('skyCEsidmesso01');
  const userSkyIdToken = messoToken || ctx.cookies.get('skyCEsidismesso01');
  if (!userSkyIdToken) {
    ctx.status = 401;
    ctx.body = {
      class: ['redirect'],
      properties: {
        targetApi: 'signin',
        type: 'redirectApi',
        eventType: 'error',
        href:
          'https://uiapi.id.func.prod.nowtv.com/signin/service/international?continuationUrl=https%3A%2F%2Fuiapi.id.func.prod.nowtv.com%2Fmodifyprofile%2Finternational%3Fapi_id%3Dmodifyprofile',
      },
    };
  } else {
    ctx.body = getModifyUserProfileFixture;
  }
};

module.exports.postModifyUserProfile = function(ctx) {
  const messoToken = ctx.cookies.get('skyCEsidmesso01');
  const userSkyIdToken = messoToken || ctx.cookies.get('skyCEsidismesso01');
  if (!userSkyIdToken) {
    ctx.status = 401;
    ctx.body = {
      class: ['redirect'],
      properties: {
        targetApi: 'signin',
        type: 'redirectApi',
        eventType: 'error',
        href:
          'https://uiapi.id.func.prod.nowtv.com/signin/service/international?continuationUrl=https%3A%2F%2Fuiapi.id.func.prod.nowtv.com%2Fmodifyprofile%2Finternational%3Fapi_id%3Dmodifyprofile',
      },
    };
  } else if (isScenario(ctx, 'modify_user_profile_500_error')) {
    ctx.status = 500;
  } else {
    const globalBookmarkingOptOut = ctx.request.body.globalBookmarkingOptOut;
    if (globalBookmarkingOptOut !== undefined) {
      ctx.persistentStore.set('globalBookmarkingOptOut', globalBookmarkingOptOut === 'true');
    }
    ctx.status = 201;
  }
};

module.exports.postActivationStatus = function(ctx) {
  const messoToken = ctx.cookies.get('skyCEsidmesso01');
  const userSkyIdToken = messoToken || ctx.cookies.get('skyCEsidismesso01');
  if (!userSkyIdToken) {
    ctx.status = 401;
    ctx.body = {
      class: ['redirect'],
      properties: {
        targetApi: 'signin',
        type: 'redirectApi',
        eventType: 'error',
        href:
          'https://uiapi.id.func.prod.nowtv.com/signin/service/international?continuationUrl=https%3A%2F%2Fuiapi.id.func.prod.nowtv.com%2Fmodifyprofile%2Factivationstatus%2Finternational%3Fapi_id%3Dmodifyprofile',
      },
    };
  } else if (ctx.request.body.state !== 'complete') {
    ctx.status = 500;
  } else {
    ctx.persistentStore.set('fullySignedUp', true);
    ctx.status = 204;
  }
};

module.exports.signInCaptcha = function(ctx) {
  const captchaId = ctx.request.body.captchaId;
  const captcha = ctx.request.body.captcha;

  const preCaptchaCredentialsCorrect = ctx.cookies.get('preCaptchaCredentialsCorrect') === 'true';
  const preCaptchaUsername = ctx.cookies.get('preCaptchaUserName');
  const isValidCaptchaSubmission = preCaptchaUsername && Boolean(captchaId);
  const isCurrentCaptchaSolutionCorrect = isCaptchaSolutionCorrect(captcha, preCaptchaUsername);

  if (isValidCaptchaSubmission && isCurrentCaptchaSolutionCorrect && preCaptchaCredentialsCorrect) {
    ctx.status = 201;
    ctx.body = setCookiesAndGetSignInSuccessResponse(ctx, preCaptchaUsername);
  } else if (isValidCaptchaSubmission && isCurrentCaptchaSolutionCorrect && !preCaptchaCredentialsCorrect) {
    ctx.status = 422;
    ctx.body = replaceLocalhostWithOrigin(signInIncorrectCredentialsResponseBody, ctx);
  } else {
    ctx.status = 422;
    ctx.body = replaceLocalhostWithOrigin(signInWrongCaptchaResponseBody, ctx);
  }
};

module.exports.signin = function(ctx) {
  const header = ctx.request.header;
  const username = ctx.request.body.userIdentifier;
  const password = ctx.request.body.password;

  if (!isCorrectContentType(header)) {
    ctx.status = 500;
  } else if (!isCorrectSkyIdHeader(header)) {
    ctx.status = 400;
  } else if (isSignInCaptchaUser(username, password)) {
    ctx.status = 200;
    setPreCaptchaCredentialsCookies(ctx, username, password);
    ctx.body = replaceLocalhostWithOrigin(signInCaptchaResponseBody, ctx);
  } else if (!isCorrectSkyIdCredentials(username, password)) {
    ctx.status = 422;
    ctx.body = replaceLocalhostWithOrigin(signInIncorrectCredentialsResponseBody, ctx);
  } else {
    ctx.status = 201;
    ctx.body = setCookiesAndGetSignInSuccessResponse(ctx, username);
  }
};

function getNewCaptcha(ctx) {
  const captcha1 = signInCaptchaDataResponseBody;
  const captcha2 = signUpCaptchaDataFixture;
  const previousCaptcha = ctx.persistentStore.get('previousCaptcha');

  let nextCaptcha = captcha1;
  if (previousCaptcha && previousCaptcha === JSON.stringify(captcha1)) {
    nextCaptcha = captcha2;
  }

  ctx.persistentStore.set('previousCaptcha', JSON.stringify(nextCaptcha));
  return nextCaptcha;
}

module.exports.signInCaptchaData = function(ctx) {
  ctx.status = 201;
  ctx.body = replaceLocalhostWithOrigin(getNewCaptcha(ctx), ctx);
};

module.exports.createProfile = function(ctx) {
  const header = ctx.request.header;
  const fixture = signUpFixture;

  if (!isCorrectSkyIdHeader(header)) {
    return (ctx.status = 400);
  }

  const validationErrors = validateCreateProfile(ctx.request.body);
  const errors = validationErrors.length > 0 ? { properties: { fieldErrors: validationErrors } } : false;

  if (isScenario(ctx, 'sign_up_submit_500_error')) {
    ctx.status = 500;
  } else if (isScenario(ctx, 'sign_up_submit_captcha_required')) {
    ctx.status = 422;
    ctx.body = signUpCaptchaFixture;
  } else if (isScenario(ctx, 'sign_up_captcha_required_but_invalid')) {
    ctx.status = 422;
    ctx.body = signUpCaptchaErrorFixture;
  } else if (errors) {
    const responseBody = Object.assign(fixture, errors.properties);
    ctx.status = 422;
    ctx.body = responseBody;
  } else if (ctx.request.body.email.match(/existing-email/)) {
    ctx.status = 422;
    ctx.body = Object.assign({}, fixture, {
      properties: {
        errors: {
          fieldErrors: [
            {
              target: 'email',
              message: "You've already created an account with this email address",
              code: 'email.taken',
            },
          ],
          categoryErrors: [],
        },
      },
    });
  } else if (ctx.request.body.email.match(/service-error/)) {
    ctx.status = 503;
    ctx.body = {
      class: ['error', 'redirectAbsolute'],
      properties: {
        eventType: 'error',
        type: 'redirectAbsolute',
        href: 'https://skyIdUrl/error/unavailable/tv',
      },
    };
  } else if (ctx.request.body.email.match(/unhandled-error/)) {
    ctx.status = 500;
  } else {
    const messoToken = Math.random().toString();
    users[ctx.request.body.email] = {
      userToken: messoToken,
      password: 'test1234',
      skyIdToken: `01-${messoToken}`,
    };
    ctx.body = setCookiesAndGetSignInSuccessResponse(ctx, ctx.request.body.email);
  }
};

module.exports.captchaAssets = function(ctx) {
  const { type, id } = ctx.params;

  const imagePath = `${__dirname}/sky-id/${id}.${type}`;
  ctx.type = path.extname(imagePath);
  ctx.body = fs.createReadStream(imagePath);
};

module.exports.signupCaptcha = function(ctx) {
  ctx.status = 201;
  ctx.body = replaceLocalhostWithOrigin(getNewCaptcha(ctx), ctx);
};

module.exports.getServiceSignUp = function(ctx) {
  ctx.status = 201;
};

module.exports.getResetPasswordForm = function(ctx) {
  if (!isCorrectSkyIdHeader(ctx.request.header)) {
    return (ctx.status = 400);
  }

  const response = replaceLocalhostWithOrigin(resetPasswordForm, ctx);

  response.actions[0].fields[lodashFindIndex(response.actions[0].fields, { name: 'resetPasswordUrl' })].value =
    ctx.request.query.resetPasswordUrl;

  ctx.body = response;
  ctx.set('Content-Type', 'application/vnd.siren+json');
};

module.exports.postResetPasswordForm = function(ctx) {
  if (!isCorrectContentType(ctx.request.header)) {
    return (ctx.status = 500);
  }

  if (
    !isCorrectSkyIdHeader(ctx.request.header) ||
    ctx.request.body._csrf !== lodashFind(resetPasswordForm.actions[0].fields, { name: '_csrf' }).value
  ) {
    return (ctx.status = 400);
  }

  let data;

  if (!ctx.request.body.userIdentifier || !ctx.request.body.resetPasswordUrl) {
    ctx.status = 422;

    data = Object.assign({}, resetPasswordForm, {
      properties: {
        errors: {
          fieldErrors: [
            !ctx.request.body.userIdentifier
              ? {
                  target: 'userIdentifier',
                  message: 'Please enter your email address',
                  code: 'userIdentifier.blank',
                }
              : {
                  target: 'resetPasswordUrl',
                  message: 'Please enter reset password url',
                  code: 'resetPasswordUrl.blank',
                },
          ],
          categoryErrors: [],
        },
      },
    });

    data.actions[0].fields[lodashFindIndex(data.actions[0].fields, { name: 'resetPasswordUrl' })].value =
      ctx.request.resetPasswordUrl;
  } else if (ctx.request.body.userIdentifier && ctx.request.body.userIdentifier.length > 100) {
    ctx.status = 422;

    data = Object.assign({}, resetPasswordForm, {
      properties: {
        errors: {
          fieldErrors: [
            {
              target: 'userIdentifier',
              message: 'Your email or username cannot be longer than 100 characters',
              code: 'userIdentifier.size.toobig',
            },
          ],
          categoryErrors: [],
        },
      },
    });

    data.actions[0].fields[lodashFindIndex(data.actions[0].fields, { name: 'resetPasswordUrl' })].value =
      ctx.request.resetPasswordUrl;
  } else {
    ctx.status = 201;
    data = resetPasswordResponse;
  }

  let response = replaceLocalhostWithOrigin(data, ctx);
  response = JSON.parse(JSON.stringify(response).replace(/__USERIDENTIFIER__/g, ctx.request.body.userIdentifier));

  ctx.body = response;
  ctx.set('Content-Type', 'application/vnd.siren+json');
};

module.exports.getNewPasswordForm = function(ctx) {
  if (!isCorrectSkyIdHeader(ctx.request.header)) {
    return (ctx.status = 400);
  }

  let data;

  if (ctx.params.token === 'expired-token') {
    ctx.status = 404;
    data = newPasswordFormExpired;
  } else {
    ctx.status = 200;
    data = newPasswordForm;
  }

  ctx.body = replaceLocalhostWithOrigin(data, ctx);
  ctx.set('Content-Type', 'application/vnd.siren+json');
};

module.exports.postNewPasswordForm = function(ctx) {
  const messoToken = ctx.cookies.get('skyCEsidmesso01');
  const userSkyIdToken = messoToken || ctx.cookies.get('skyCEsidismesso01');
  const user = Object.values(users).find(u => u.skyIdToken === userSkyIdToken) || users.valid_user;

  if (!isCorrectContentType(ctx.request.header)) {
    return (ctx.status = 500);
  }

  if (
    !isCorrectSkyIdHeader(ctx.request.header) ||
    !ctx.request.body._csrf ||
    !ctx.request.body.enterNewPasswordId ||
    !ctx.request.body.password ||
    !ctx.request.body.confirmPassword
  ) {
    return (ctx.status = 400);
  }

  let data;

  if (ctx.request.body.password === 'csrf_expired1') {
    ctx.status = 412;
    data = newPasswordResponseExpired;
  } else if (ctx.request.body.password === 'alreadyused1') {
    ctx.status = 400;
    data = newPasswordResponseAlreadyUsed;
  } else if (ctx.request.body.password === user.username || ctx.request.body.password.includes('name')) {
    ctx.status = 422;
    data = Object.assign({}, newPasswordForm, {
      properties: {
        errors: {
          fieldErrors: [
            {
              target: 'password',
              message:
                "Please enter a valid password, it can't contain your name, email address or the word 'password'",
              code: 'password.contains.firstName',
            },
          ],
          categoryErrors: [],
        },
      },
    });
  } else if (
    ctx.request.body.password.length < 8 ||
    ctx.request.body.password.length > 32 ||
    ctx.request.body.password !== ctx.request.body.confirmPassword
  ) {
    ctx.status = 422;
    data = Object.assign({}, newPasswordForm, {
      properties: {
        errors: {
          fieldErrors: [
            // all the errors
            {
              target: 'password',
              message:
                "Please enter a valid password, it can't contain your name, email address or the word 'password'",
              code: 'password.invalid',
            },
            {
              target: 'password',
              message: 'Your password can only have 8 - 32 characters',
              code: 'password.size.toosmall',
            },
            {
              target: 'password',
              message: 'Your password must contain letters and numbers',
              code: 'password.must.contain.number',
            },
            {
              target: 'password',
              message: 'Your password can only have 8 - 32 characters',
              code: 'password.size.toosmall',
            },
            {
              target: 'password',
              message:
                "Please enter a valid password, it can't contain your name, email address or the word 'password'",
              code: 'password.contains.firstName',
            },
            {
              target: 'password',
              message:
                "Please enter a valid password, it can't contain your name, email address or the word 'password'",
              code: 'password.contains.lastName',
            },
            {
              target: 'confirmPassword',
              message: "The passwords you entered didn't match. Please try again",
              code: 'confirmPassword.must.match.password',
            },
          ],
          categoryErrors: [],
        },
      },
    });
  } else {
    ctx.status = 201;
    data = newPasswordResponse;
  }

  ctx.body = replaceLocalhostWithOrigin(data, ctx);
  ctx.set('Content-Type', 'application/vnd.siren+json');
};

module.exports.newPasswordEmail = async function(ctx) {
  const { returnUrl, email } = ctx.params;
  const { territory } = ctx.query;
  const resetPasswordToken = email === 'expired-token' ? 'expired-token' : email + 'A'.repeat(64 - email.length);
  let pathName = 'reset-password';

  if (territory.toLowerCase() === 'es') {
    pathName = 'restablecer-contrasena';
  } else if (territory.toLowerCase() === 'at') {
    pathName = 'passwort-zuruecksetzen';
  }

  await ctx.render('reset-password-email', { resetPasswordToken, origin: returnUrl, path: pathName });
};

module.exports.resetFixtures = function(ctx) {
  ctx.persistentStore.delete('previousCaptcha');
  ctx.persistentStore.delete('invalidate-login');
  ctx.persistentStore.delete('fullySignedUp');
  ctx.persistentStore.delete('globalBookmarkingOptOut');

  ctx.body = `Fixtures for sky id have been reset for user: ${ctx.request.query.userToken}`;
  ctx.status = 200;
};

module.exports.createClone = function(ctx) {
  const cloneId = ctx.request.body.id;
  createCloneForAllUsers(cloneId, ctx.entirePersistentStore);
  ctx.body = `Clone ${cloneId} for all users has been created`;
  ctx.status = 200;
};

module.exports.removeClone = function(ctx) {
  const cloneId = ctx.params.id;
  removeCloneForAllUsers(cloneId);
  ctx.body = `Clone ${cloneId} for all users has been removed`;
  ctx.status = 200;
};

module.exports.getExternalResetPasswordPage = async function(ctx) {
  await ctx.render('external-reset-password', { successUrl: ctx.request.query.successUrl });
};
