const { intersectionBy, cloneDeep } = require('lodash');

const users = require('../sky-id/sky-id/users');
const entitlements = require('../commerce/entitlements');
const { isScenario } = require('../../utils/scenario');
const userIdentifier = require('../../utils/user-identifier');

const userDetails = require('./umv/v1/user-details');
const v2UserDetails = require('./umv/v2/user-details');

const ACTIVATED = 'ACTIVATED';
const QUEUED = 'QUEUED';

function validMessoPredicate(messoToken) {
  return username => {
    const userMessoToken = users[username].skyIdToken;
    const userIsMessoToken = users[username].skyIdToken;

    if (userMessoToken === messoToken || userIsMessoToken === messoToken) {
      return username;
    }
  };
}

function modifyUserAccount(user, entitlementsForUser, accountSegmentsForUser) {
  const productsSettingAccountSegments = { C_006602: { name: 'HD' } };
  const accountSegmentProduct =
    entitlementsForUser.entitlements &&
    entitlementsForUser.entitlements.find(product =>
      Object.keys(productsSettingAccountSegments).includes(product.productId)
    );

  if (accountSegmentsForUser) {
    user.segmentation.account = accountSegmentsForUser;
  }
  if (accountSegmentProduct) {
    user.segmentation.account = intersectionBy(
      user.segmentation.account ? user.segmentation.account.concat(accountSegmentProduct) : [accountSegmentProduct],
      'productId'
    );
  }
}

function getUserDetailsWithEntitlements(userInfo, entitlementsForUser, accountSegmentsForUser) {
  const clonedUserInfo = cloneDeep(userInfo);

  if (entitlementsForUser.entitlements && entitlementsForUser.entitlements.length > 0) {
    const content = entitlementsForUser.entitlements.map(entitlement => ({
      name: entitlement.name,
      state: entitlement.state,
    }));

    if (clonedUserInfo && clonedUserInfo.segmentation && clonedUserInfo.segmentation.content) {
      clonedUserInfo.segmentation.content = content;
    } else {
      clonedUserInfo.segmentation = {
        content,
      };
    }
  } else {
    delete clonedUserInfo.segmentation;
  }

  modifyUserAccount(clonedUserInfo, entitlementsForUser, accountSegmentsForUser);
  return clonedUserInfo;
}

function getV2UserDetailsWithEntitlements(userInfo, entitlementsForUser, accountSegmentsForUser) {
  const clonedUserInfo = cloneDeep(userInfo);
  const userEntitlements =
    entitlementsForUser.entitlements &&
    entitlementsForUser.entitlements.map(entitlement => {
      const baseEntitlement = {
        name: entitlement.name,
        state: entitlement.state,
        endDate: entitlement.endDate,
      };

      if (entitlement.state === QUEUED || (entitlement.state === ACTIVATED && entitlement.endDate)) {
        return {
          ...baseEntitlement,
          indicativeContentSegments: entitlement.contentSegments.map(segment => ({ name: segment })),
        };
      }

      return baseEntitlement;
    });

  const entitlementNames =
    userEntitlements &&
    userEntitlements
      .filter(entitlement => entitlement.state === ACTIVATED)
      .map(entitlement => ({ name: entitlement.name }));

  if (clonedUserInfo.segmentation && clonedUserInfo.segmentation.content) {
    clonedUserInfo.segmentation.content = entitlementNames;
  } else {
    clonedUserInfo.segmentation = {
      content: entitlementNames,
    };
  }

  clonedUserInfo.entitlements = userEntitlements;

  modifyUserAccount(clonedUserInfo, entitlementsForUser, accountSegmentsForUser);

  if (entitlementsForUser && entitlementsForUser.entitlements && entitlementsForUser.entitlements.length === 0) {
    clonedUserInfo.segmentation.account = [];
  }

  return clonedUserInfo;
}

module.exports.getUserToken = function(ctx) {
  const validUsernames = Object.keys(users);
  const messoToken = ctx.request.body.auth.authToken;
  const usernameWithSSO = validUsernames.find(validMessoPredicate(messoToken));
  if (messoToken && messoToken.includes('01-uk_user')) {
    ctx.status = 400;
    ctx.body = {
      errorCode: 'OVP_00026',
      description: 'Required terms and conditions not accepted',
    };
  } else if (usernameWithSSO && users[usernameWithSSO]) {
    ctx.status = 200;
    ctx.body = {
      userToken: users[usernameWithSSO].userToken,
    };
  } else {
    ctx.status = 403;
    ctx.body = {
      errorCode: 'OVP_00200',
      description: 'Invalid authentication token',
    };
    ctx.set('Content-Type', 'application/json');
  }
};

module.exports.getBTLinkedUserToken = function(ctx) {
  const contentType = 'application/vnd.bridge.v1+json';
  const hasContentType = ctx.get('Content-Type') === contentType;
  const hasAccept = ctx.get('Content-Type') === contentType;
  const hasCertificate = ctx.get('SSLClientCertHash') === 'a-certificate';

  const validUsernames = Object.keys(users);
  const body = JSON.parse(ctx.request.rawBody);
  const username = body.username === body.password && body.password;
  const messoToken = users[username].skyIdToken;
  const usernameWithSSO = validUsernames.find(validMessoPredicate(messoToken));
  if (!hasCertificate) {
    ctx.status = 400;
    ctx.body = {
      message: 'Invalid headers',
      service: 'oauthgateway',
    };
  } else if (!hasContentType) {
    ctx.status = 415;
    ctx.body = {
      message: 'Content-Type value not supported',
      service: 'oauthgateway',
    };
  } else if (!hasAccept) {
    ctx.status = 406;
    ctx.body = {
      message: 'Invalid Accept header: */*',
      service: 'oauthgateway',
    };
  } else if (!username || !messoToken) {
    ctx.status = 404;
    ctx.body = {
      message: 'Partner account does not exist PA_0201',
      service: 'Partner Gateway',
    };
  } else if (usernameWithSSO && users[usernameWithSSO]) {
    ctx.status = 200;
    ctx.body = {
      access_token: messoToken,
      token_type: 'token',
    };
  }
};

const userDetailsVersion = {
  'application/vnd.userinfo.v1+json': (userEntitlements, purchasedAccountSegments) =>
    getUserDetailsWithEntitlements(userDetails, userEntitlements, purchasedAccountSegments),
  'application/vnd.userinfo.v2+json': (userEntitlements, purchasedAccountSegments) =>
    getV2UserDetailsWithEntitlements(v2UserDetails, userEntitlements, purchasedAccountSegments),
};

module.exports.getUserDetails = function(ctx) {
  // Temp issue with devices not sending content-type to mock proxy so use the Accept header
  const authMeVersion = ctx.get('Accept');
  if (isScenario(ctx, 'user_details_500_error')) {
    ctx.status = 500;
  } else if (isScenario(ctx, 'user_details_400_error')) {
    ctx.status = 400;
  } else if (isScenario(ctx, 'user_details_403_error')) {
    ctx.status = 403;
  } else if (isScenario(ctx, 'user_details_406_error')) {
    ctx.status = 406;
  } else {
    ctx.status = 200;
    ctx.body = userDetailsVersion[authMeVersion](
      entitlements.getEntitlementsForUser(ctx),
      ctx.persistentStore.get('purchasedAccountSegments')
    );
    ctx.set('Content-Type', 'application/json');
  }
};

module.exports.getUserParentalControl = function(ctx) {
  const user = userIdentifier.getDataForUserOrDefault(ctx);

  ctx.status = 200;
  ctx.set('Content-Type', 'application/vnd.parental-control.v1+json');
  ctx.body = {
    rating: user.parentalPin.parentalPinRequired ? '15' : 'None',
    hashedPin: '6ZBy6tUoIGjxeeKVAmc7XL3dNrKunBB8EAxCcbjn1f0=',
    lastModifiedDate: '2016-11-15T13:14:15.000Z',
  };

  if (isScenario(ctx, 'get_user_parental_controls_500_error')) {
    ctx.status = 500;
    ctx.body = {};
  }
};
