'use strict';

const sinon = require('sinon');
const supertest = require('supertest');

const app = require('../app');
const accountDetailsFixture = require('../routes/commerce/commerce/account-manager/account-details');
const parentalControlOffFixture = require('../routes/commerce/commerce/account-manager/parental-control/turned-off');
const parentalControlOnFixture = require('../routes/commerce/commerce/account-manager/parental-control/turned-on');
const parentalControlNeverSetFixture = require('../routes/commerce/commerce/account-manager/parental-control/never-set');

const HEADERS = {
  'X-SkyOTT-Device': 'COMPUTER',
  'X-SkyOTT-Platform': 'PC',
  'X-SkyOTT-Territory': 'IE',
  'X-SkyOTT-Proposition': 'NOWTV',
  'X-SkyOTT-Provider': 'NOWTV',
  'Content-Type': 'application/vnd.account-manager.v1+json',
};

const CORRECT_ACCOUNT_GET_HEADERS = Object.assign({}, HEADERS, {
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
  'X-SkyOTT-UserToken': '1234567',
});

const CORRECT_ACCOUNT_PUT_HEADERS = Object.assign({}, HEADERS, {
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
  'X-SkyOTT-UserToken': '1234567',
});

const CORRECT_PIN_GET_HEADERS = Object.assign({}, HEADERS, {
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
  'X-SkyOTT-UserToken': '1234567',
});

const CORRECT_PIN_PUT_HEADERS = Object.assign({}, HEADERS, {
  'Content-MD5': 'c18cf7e783890da74e38b42aa06f2b92',
  'X-SkyOTT-UserToken': '1234567',
});

const HEADERS_WITH_INVALID_SIGNATURE = Object.assign({}, HEADERS, {
  'Content-MD5': 'invalid',
  'X-SkyOTT-UserToken': '1234567',
});

const ACCOUNT_DETAILS_URL = '/commerce/account-manager/account';
const PARENTAL_CONTROL_URL = '/commerce/account-manager/account/parental-control';

describe('Account Manager', () => {
  let clock;
  let server;

  beforeAll(() => {
    clock = sinon.useFakeTimers(1457621019000);
    server = app.listen();
  });

  afterAll(() => {
    clock.restore();
    server.close();
  });

  describe('/commerce/account-manager/account', () => {
    describe('GET', () => {
      it('should return 200 and response body containing account details', () => {
        return supertest(server)
          .get(ACCOUNT_DETAILS_URL)
          .set(CORRECT_ACCOUNT_GET_HEADERS)
          .expect(200, accountDetailsFixture);
      });

      it('should return 400 if the user token header is not set', () => {
        return supertest(server)
          .get(ACCOUNT_DETAILS_URL)
          .set(HEADERS)
          .expect(400, {
            errorCode: 'COM_00002',
            description: 'Invalid ott user token provided',
          });
      });

      it('should return 403 if the signature is not valid', () => {
        return supertest(server)
          .get(ACCOUNT_DETAILS_URL)
          .set(HEADERS_WITH_INVALID_SIGNATURE)
          .expect(403);
      });
    });

    describe('PUT', () => {
      it('should return 204', () => {
        return supertest(server)
          .put(ACCOUNT_DETAILS_URL)
          .set(CORRECT_ACCOUNT_PUT_HEADERS)
          .expect(204);
      });

      it('should return 400 if the user token header is not set', () => {
        return supertest(server)
          .put(ACCOUNT_DETAILS_URL)
          .set(HEADERS)
          .expect(400, {
            errorCode: 'COM_00002',
            description: 'Invalid ott user token provided',
          });
      });

      it('should return 403 if the signature is not valid', () => {
        return supertest(server)
          .put(ACCOUNT_DETAILS_URL)
          .set(HEADERS_WITH_INVALID_SIGNATURE)
          .expect(403);
      });
    });
  });

  describe('/commerce/account-manager/account/parental-control', () => {
    describe('GET', () => {
      it('should return 200 and response body containing never set parental controls', () => {
        return supertest(server)
          .get(PARENTAL_CONTROL_URL)
          .set({
            'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
            'X-SkyOTT-UserToken': 'user_never_set_parental_pin',
          })
          .expect(200, parentalControlNeverSetFixture);
      });

      it('should return 200 and response body containing turned on parental controls', () => {
        return supertest(server)
          .get(PARENTAL_CONTROL_URL)
          .set(CORRECT_PIN_GET_HEADERS)
          .expect(200, parentalControlOnFixture);
      });

      it('should return 200 and response body containing turned off parental controls', () => {
        return supertest(server)
          .get(PARENTAL_CONTROL_URL)
          .set(CORRECT_PIN_GET_HEADERS)
          .set({
            'X-SkyOTT-UserToken': 'user_without_parental_pin',
          })
          .expect(200, parentalControlOffFixture);
      });

      it('should return 400 if the user token header is not set', () => {
        return supertest(server)
          .get(PARENTAL_CONTROL_URL)
          .set(HEADERS)
          .expect(400, {
            errorCode: 'COM_00002',
            description: 'Invalid ott user token provided',
          });
      });

      it('should return 403 if the signature is not valid', () => {
        return supertest(server)
          .get(PARENTAL_CONTROL_URL)
          .set(HEADERS_WITH_INVALID_SIGNATURE)
          .expect(403);
      });
    });

    describe('PUT', () => {
      it('should return 204', () => {
        return supertest(server)
          .put(PARENTAL_CONTROL_URL)
          .set(CORRECT_PIN_PUT_HEADERS)
          .send({ rating: '18' })
          .expect(204);
      });

      it('should return 400 if the user token header is not set', () => {
        return supertest(server)
          .put(PARENTAL_CONTROL_URL)
          .set(HEADERS)
          .expect(400, {
            errorCode: 'COM_00002',
            description: 'Invalid ott user token provided',
          });
      });

      it('should return 403 if the signature is not valid', () => {
        return supertest(server)
          .put(PARENTAL_CONTROL_URL)
          .set(HEADERS_WITH_INVALID_SIGNATURE)
          .expect(403);
      });
    });

    it('should use PUT data in subsequent GET', () => {
      return supertest(server)
        .put(PARENTAL_CONTROL_URL)
        .set(CORRECT_PIN_PUT_HEADERS)
        .send({ rating: '18' })
        .expect(204)
        .then(() => {
          return supertest(server)
            .get(PARENTAL_CONTROL_URL)
            .set(CORRECT_PIN_GET_HEADERS)
            .expect(
              200,
              Object.assign({}, parentalControlOnFixture, {
                rating: '18',
                lastModifiedDate: '2016-03-10T14:43:39.000Z',
              })
            );
        });
    });

    beforeEach(() => {
      return supertest(server)
        .get('/reset-fixtures/account/parental-control')
        .set(CORRECT_PIN_GET_HEADERS)
        .expect(200);
    });
  });
});
