'use strict';

const supertest = require('supertest');
const sinon = require('sinon');

const users = require('../routes/sky-id/sky-id/users');
const app = require('../app');

describe('Payment Accounts', () => {
  let clock;
  let server;

  beforeAll(() => {
    clock = sinon.useFakeTimers(1457621019000);
    server = app.listen();
  });

  afterAll(() => {
    clock.restore();
    server.close();
  });

  it('should return 403 if the MD5 hash is not valid', () => {
    return supertest(server)
      .get('/commerce/bridge/user/paymentaccounts')
      .set({
        'X-SkyOTT-Device': 'COMPUTER',
        'X-SkyOTT-Platform': 'PC',
        'X-SkyOTT-Territory': 'IE',
        'X-SkyOTT-Proposition': 'NOWTV',
        'X-SkyOTT-Provider': 'NOWTV',
        'X-SkyOTT-UserToken': '1234567',
        'Content-Type': 'application/vnd.paymentaccounts.v1+json',
        'Content-MD5': 'INVALID',
      })
      .expect(403, {
        errorCode: 'COM_30000',
        description: 'Security failure',
      });
  });

  it('should return 400 if the user token header is not set', () => {
    return supertest(server)
      .get('/commerce/bridge/user/paymentaccounts')
      .set({
        'X-SkyOTT-Device': 'COMPUTER',
        'X-SkyOTT-Platform': 'PC',
        'X-SkyOTT-Territory': 'IE',
        'X-SkyOTT-Proposition': 'NOWTV',
        'X-SkyOTT-Provider': 'NOWTV',
        'Content-Type': 'application/vnd.paymentaccounts.v1+json',
        'Content-MD5': '88adc50782a64ccf8c5125eb395329d2',
      })
      .expect(400, {
        errorCode: 'COM_00002',
        description: 'Invalid ott user token provided',
      });
  });

  describe('GET /commerce/bridge/user/paymentaccounts', () => {
    it('should return 200 if the correct headers are sent', () => {
      const expectedPaymentAccountJson = users.valid_user.paymentAccounts;

      return supertest(server)
        .get('/commerce/bridge/user/paymentaccounts')
        .set({
          'X-SkyOTT-Device': 'COMPUTER',
          'X-SkyOTT-Platform': 'PC',
          'X-SkyOTT-Territory': 'IE',
          'X-SkyOTT-Proposition': 'NOWTV',
          'X-SkyOTT-Provider': 'NOWTV',
          'X-SkyOTT-UserToken': '1234567',
          'Content-Type': 'application/vnd.paymentaccounts.v1+json',
          'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
        })
        .expect('Content-Type', 'application/vnd.paymentaccounts.v1+json')
        .expect('X-SkyOTT-UserToken', '1234567')
        .expect(200, expectedPaymentAccountJson);
    });
  });

  describe('POST /commerce/payments-manager/user/paymentaccount', () => {
    it('should return 204 if the correct headers and body is sent', () => {
      return supertest(server)
        .post('/commerce/payments-manager/user/paymentaccount')
        .set({
          'X-SkyOTT-Device': 'COMPUTER',
          'X-SkyOTT-Platform': 'PC',
          'X-SkyOTT-Territory': 'IE',
          'X-SkyOTT-Proposition': 'NOWTV',
          'X-SkyOTT-Provider': 'NOWTV',
          'X-SkyOTT-UserToken': '1234567',
          'Content-Type': 'application/vnd.paymentaccount.v1+json',
          'Content-MD5': '8b46659ebf69a35f04f49cfd6a9559fe',
        })
        .send({ some: 'content' })
        .expect('X-SkyOTT-UserToken', '1234567')
        .expect(204);
    });
  });
});
