'use strict';

const supertest = require('supertest');
const sinon = require('sinon');

const app = require('../app');
const users = require('../routes/sky-id/sky-id/users');

const BASE_HEADERS = {
  'X-SkyOTT-Agent': 'skygo.computer.pc',
  'X-SkyOTT-Device': 'COMPUTER',
  'X-SkyOTT-Platform': 'PC',
  'X-SkyOTT-Territory': 'GB',
  'X-SkyOTT-Proposition': 'SKYGO',
  'X-SkyOTT-Provider': 'SKY',
  'Content-Length': 0,
};

const USER_TOKEN_HEADER = { 'X-SkyOTT-UserToken': users.valid_user.userToken };
const USER_WITHOUT_PAYMENT_PIN_USER_TOKEN_HEADER = { 'X-SkyOTT-UserToken': users.user_without_payment_pin.userToken };

const CORRECT_HEADERS_FOR_PIN_REQUIRED = Object.assign({}, BASE_HEADERS, USER_TOKEN_HEADER, {
  Accept: 'application/vnd.pinrequired.v1+json',
  'Content-Type': 'application/vnd.pinrequired.v1+json',
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
});

const CORRECT_HEADERS_FOR_NO_PAYMENT_PIN = Object.assign({}, BASE_HEADERS, USER_WITHOUT_PAYMENT_PIN_USER_TOKEN_HEADER, {
  Accept: 'application/vnd.pinrequired.v1+json',
  'Content-Type': 'application/vnd.pinrequired.v1+json',
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
});

const HEADERS_WITH_INVALID_SIGNATURE = Object.assign({}, BASE_HEADERS, USER_TOKEN_HEADER, {
  'Content-MD5': 'invalid',
});

const HEADERS_WITH_USER_TOKEN_NOT_SET = Object.assign({}, BASE_HEADERS, {
  'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
});

const INVALID_USER_TOKEN_ERROR = {
  errorCode: 'COM_00002',
  description: 'Invalid ott user token provided',
};

const SECURITY_FAILURE_ERROR = {
  errorCode: 'COM_30000',
  description: 'Security failure',
};

const PAYMENT_PIN_NOT_FOUND = {
  errorCode: 'COM_00005',
  description: 'Payment Pin not found',
};

describe('Payment Pin', () => {
  let clock;
  let server;

  beforeAll(() => {
    clock = sinon.useFakeTimers(1457621019000);
    server = app.listen();
  });

  afterAll(() => {
    clock.restore();
    server.close();
  });

  describe('GET /commerce/bridge/user/paymentpin/pinrequired', () => {
    it('should return 200 and pin details if the correct headers are set', () => {
      const expectedPinJson = { pinRequired: true };
      return supertest(server)
        .get('/commerce/bridge/user/paymentpin/pinrequired')
        .set(CORRECT_HEADERS_FOR_PIN_REQUIRED)
        .expect('X-SkyOTT-UserToken', users.valid_user.userToken)
        .expect(200, expectedPinJson);
    });

    it('should return 404 and pin not found response if the user does not have a pin', () => {
      return supertest(server)
        .get('/commerce/bridge/user/paymentpin/pinrequired')
        .set(CORRECT_HEADERS_FOR_NO_PAYMENT_PIN)
        .expect('X-SkyOTT-UserToken', users.user_without_payment_pin.userToken)
        .expect(404, PAYMENT_PIN_NOT_FOUND);
    });

    it('should return 403 if the signature is not valid', () => {
      return supertest(server)
        .get('/commerce/bridge/user/paymentpin/pinrequired')
        .set(HEADERS_WITH_INVALID_SIGNATURE)
        .expect(403, SECURITY_FAILURE_ERROR);
    });

    it('should return 400 if the user token header is not set', () => {
      return supertest(server)
        .get('/commerce/bridge/user/paymentpin/pinrequired')
        .set(HEADERS_WITH_USER_TOKEN_NOT_SET)
        .expect(400, INVALID_USER_TOKEN_ERROR);
    });
  });
});
