const superagent = require('supertest');
const sinon = require('sinon');

const app = require('../app');

describe('Payments Manager', () => {
  function updateCancelledSubscriptionsDate(subscription) {
    if (subscription.cancellationEffectiveDate) {
      return Object.assign({}, subscription, {
        cancellationEffectiveDate: '2016-03-14T14:43:39.000Z',
      });
    }

    return subscription;
  }

  const endpoints = {
    '/commerce/payments-manager/entitlements': {
      method: 'get',
      type: 'application/vnd.entitlements.v1+json',
      requestSignature: 'd41d8cd98f00b204e9800998ecf8427e',
      response: {
        entitlements: require('../routes/sky-id/sky-id/users').valid_user.entitlements,
      },
    },
    '/commerce/payments-manager/subscriptions/latest': {
      method: 'get',
      type: 'application/vnd.latestsubscriptions.v1+json',
      requestSignature: 'd41d8cd98f00b204e9800998ecf8427e',
      response: {
        subscriptions: require('../routes/sky-id/sky-id/users').valid_user.subscriptions.map(
          updateCancelledSubscriptionsDate
        ),
      },
    },
    '/commerce/payments-manager/cancelrenewal': {
      method: 'post',
      type: 'application/vnd.cancelrenewal.v1+json',
      data: {
        subscriptionId: '15be900a-5e15-11e5-9d70-feff819cdc8f',
      },
      requestSignature: '6ee73fc4e3b8d3021b3844736392ecd5',
      response: require('../routes/commerce/commerce/payments-manager/cancelrenewal'),
    },
    '/commerce/payments-manager/reinstatesubscription': {
      method: 'post',
      type: 'application/vnd.reinstate.v1+json',
      data: {
        subscriptionId: '15be900a-5e15-11e5-9d70-feff819cdc8f',
      },
      requestSignature: '6ee73fc4e3b8d3021b3844736392ecd5',
      response: require('../routes/commerce/commerce/payments-manager/reinstatesubscription'),
    },
  };

  let clock;
  let server;

  beforeAll(() => {
    clock = sinon.useFakeTimers(1457621019000);
    server = app.listen();
  });

  afterAll(() => {
    clock.restore();
    server.close();
  });

  describe('security on endpoints', () => {
    Object.keys(endpoints).forEach(endpoint => {
      const details = endpoints[endpoint];

      describe(`${endpoint}`, () => {
        it('should return 403 if the signature is not valid', () => {
          return superagent(server)
            [details.method](endpoint)
            .set({
              'X-SkyOTT-Device': 'COMPUTER',
              'X-SkyOTT-Platform': 'PC',
              'X-SkyOTT-Territory': 'IE',
              'X-SkyOTT-Proposition': 'NOWTV',
              'X-SkyOTT-Provider': 'NOWTV',
              'X-SkyOTT-UserToken': 'valid_user',
              'Content-Type': details.type,
              'Content-MD5': 'invalid',
            })
            .expect(403, {
              errorCode: 'COM_30000',
              description: 'Security failure',
            });
        });

        it('should return 400 if the user token header is not set', () => {
          return superagent(server)
            [details.method](endpoint)
            .set({
              'X-SkyOTT-Device': 'COMPUTER',
              'X-SkyOTT-Platform': 'PC',
              'X-SkyOTT-Territory': 'IE',
              'X-SkyOTT-Proposition': 'NOWTV',
              'X-SkyOTT-Provider': 'NOWTV',
              'Content-Type': details.type,
            })
            .send(details.data)
            .expect(400, {
              errorCode: 'COM_00002',
              description: 'Invalid ott user token provided',
            });
        });

        it('should return 200 if the correct headers are sent', () => {
          return superagent(server)
            [details.method](endpoint)
            .set({
              'X-SkyOTT-Device': 'COMPUTER',
              'X-SkyOTT-Platform': 'PC',
              'X-SkyOTT-Territory': 'IE',
              'X-SkyOTT-Proposition': 'NOWTV',
              'X-SkyOTT-Provider': 'NOWTV',
              'X-SkyOTT-UserToken': 'valid_user',
              'Content-Type': details.type,
              'Content-MD5': details.requestSignature,
            })
            .send(details.data)
            .expect(200, details.response)
            .expect('Content-Type', details.type)
            .expect('X-SkyOTT-UserToken', 'valid_user');
        });
      });
    });
  });

  describe('specific functionality', () => {
    describe('GET /entitlements', () => {
      it('should return data based on the correct user', () => {
        return superagent(server)
          .get('/commerce/payments-manager/entitlements')
          .set({
            'X-SkyOTT-Device': 'COMPUTER',
            'X-SkyOTT-Platform': 'PC',
            'X-SkyOTT-Territory': 'IE',
            'X-SkyOTT-Proposition': 'NOWTV',
            'X-SkyOTT-Provider': 'NOWTV',
            'X-SkyOTT-UserToken': 'user_with_no_passes',
            'Content-Type': 'application/vnd.entitlements.v1+json',
            'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
          })
          .expect(200, {});
      });
    });

    describe('GET /subscriptions/latest', () => {
      it('should return data based on the correct user', () => {
        return superagent(server)
          .get('/commerce/payments-manager/subscriptions/latest')
          .set({
            'X-SkyOTT-Device': 'COMPUTER',
            'X-SkyOTT-Platform': 'PC',
            'X-SkyOTT-Territory': 'IE',
            'X-SkyOTT-Proposition': 'NOWTV',
            'X-SkyOTT-Provider': 'NOWTV',
            'X-SkyOTT-UserToken': 'user_with_no_passes',
            'Content-Type': 'application/vnd.latestsubscriptions.v1+json',
            'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
          })
          .expect(200, {
            subscriptions: require('../routes/sky-id/sky-id/users').user_with_no_passes.subscriptions.map(
              updateCancelledSubscriptionsDate
            ),
          });
      });

      it('should not return a cancelled subscription', () => {
        return superagent(server)
          .post('/commerce/payments-manager/cancelrenewal')
          .set({
            'X-SkyOTT-Device': 'COMPUTER',
            'X-SkyOTT-Platform': 'PC',
            'X-SkyOTT-Territory': 'IE',
            'X-SkyOTT-Proposition': 'NOWTV',
            'X-SkyOTT-Provider': 'NOWTV',
            'X-SkyOTT-UserToken': 'valid_user',
            'Content-Type': 'application/vnd.cancelrenewal.v1+json',
            'Content-MD5': 'fa317a7fe42a7329a3dce8a219220f37',
          })
          .send({
            subscriptionId: '18c4ce82-6e12-435d-9151-221efeb12af0',
          })
          .expect(200)
          .then(() => {
            return superagent(server)
              .get('/commerce/payments-manager/subscriptions/latest')
              .set({
                'X-SkyOTT-Device': 'COMPUTER',
                'X-SkyOTT-Platform': 'PC',
                'X-SkyOTT-Territory': 'IE',
                'X-SkyOTT-Proposition': 'NOWTV',
                'X-SkyOTT-Provider': 'NOWTV',
                'X-SkyOTT-UserToken': 'valid_user',
                'Content-Type': 'application/vnd.latestsubscriptions.v1+json',
                'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
              })
              .expect(200, {
                subscriptions: require('../routes/sky-id/sky-id/users')
                  .valid_user.subscriptions.map(updateCancelledSubscriptionsDate)
                  .map(subscription => {
                    if (subscription.id === '18c4ce82-6e12-435d-9151-221efeb12af0') {
                      return Object.assign({}, subscription, {
                        allowedOperations: ['REINSTATE', 'DELETE'],
                        state: 'USER_CANCELLED',
                        cancellationEffectiveDate: '2016-03-14T14:43:39.000Z',
                      });
                    }

                    return subscription;
                  }),
              });
          });
      });

      it('should allow cancelled subscriptions to be reinstated', () => {
        return superagent(server)
          .post('/commerce/payments-manager/cancelrenewal')
          .set({
            'X-SkyOTT-Device': 'COMPUTER',
            'X-SkyOTT-Platform': 'PC',
            'X-SkyOTT-Territory': 'IE',
            'X-SkyOTT-Proposition': 'NOWTV',
            'X-SkyOTT-Provider': 'NOWTV',
            'X-SkyOTT-UserToken': 'valid_user',
            'Content-Type': 'application/vnd.cancelrenewal.v1+json',
            'Content-MD5': 'fa317a7fe42a7329a3dce8a219220f37',
          })
          .send({
            subscriptionId: '18c4ce82-6e12-435d-9151-221efeb12af0',
          })
          .expect(200)
          .then(() => {
            return superagent(server)
              .post('/commerce/payments-manager/reinstatesubscription')
              .set({
                'X-SkyOTT-Device': 'COMPUTER',
                'X-SkyOTT-Platform': 'PC',
                'X-SkyOTT-Territory': 'IE',
                'X-SkyOTT-Proposition': 'NOWTV',
                'X-SkyOTT-Provider': 'NOWTV',
                'X-SkyOTT-UserToken': 'valid_user',
                'Content-Type': 'application/vnd.reinstate.v1+json',
                'Content-MD5': 'fa317a7fe42a7329a3dce8a219220f37',
              })
              .send({
                subscriptionId: '18c4ce82-6e12-435d-9151-221efeb12af0',
              })
              .expect(200)
              .then(() => {
                return superagent(server)
                  .get('/commerce/payments-manager/subscriptions/latest')
                  .set({
                    'X-SkyOTT-Device': 'COMPUTER',
                    'X-SkyOTT-Platform': 'PC',
                    'X-SkyOTT-Territory': 'IE',
                    'X-SkyOTT-Proposition': 'NOWTV',
                    'X-SkyOTT-Provider': 'NOWTV',
                    'X-SkyOTT-UserToken': 'valid_user',
                    'Content-Type': 'application/vnd.latestsubscriptions.v1+json',
                    'Content-MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                  })
                  .expect(200, {
                    subscriptions: require('../routes/sky-id/sky-id/users').valid_user.subscriptions.map(
                      updateCancelledSubscriptionsDate
                    ),
                  });
              });
          });
      });
    });
  });
});
