'use strict';

const supertest = require('supertest');

const app = require('../app');
const users = require('../routes/sky-id/sky-id/users');

const BASE_HEADERS = {
  'X-SkyOTT-Territory': 'GB',
  'X-SkyOTT-Provider': 'test',
  origin: 'http://local.nowtv.com:3000',
};

const CORRECT_HEADERS = {
  'X-SkyOTT-Territory': 'GB',
  'X-SkyOTT-Provider': 'test',
  'Content-Type': 'application/x-www-form-urlencoded',
  origin: 'http://local.nowtv.com:3000',
};

const MISSING_ORIGIN_HEADER = {
  'X-SkyOTT-Territory': 'GB',
  'X-SkyOTT-Provider': 'test',
  'Content-Type': 'application/x-www-form-urlencoded',
};

const ONE_MISSING_HEADER = {
  'X-SkyOTT-Territory': 'GB',
  'Content-Type': 'application/x-www-form-urlencoded',
  origin: 'http://local.nowtv.com:3000',
};

const INCORRECT_CONTENT_TYPE = {
  'X-SkyOTT-Territory': 'GB',
  'X-SkyOTT-Provider': 'test',
  'Content-Type': 'text/plain',
  origin: 'http://local.nowtv.com:3000',
};

const CORRECT_CREDENTIALS = {
  userIdentifier: 'valid_user',
  password: 'test1234',
};

const CORRECT_SIGN_UP_FORM = {
  title: 'Mr',
  firstName: 'Chris',
  lastName: 'Ward',
  email: 'chris@chrisss.com',
  confirmEmail: 'chris@chrisss.com',
  password: 'password',
  terms: 'true',
  marketingConsent: 'true',
  captcha: 'vfchq',
};

const INCORRECT_EMPTY_SIGN_UP_FORM = {
  title: '',
  firstName: '',
  lastName: '',
  email: '',
  confirmEmail: '',
  password: '',
  terms: '',
  marketingConsent: '',
  captcha: '',
};

const INCORRECT_INVALID_SIGN_UP_FORM = {};

const OTHER_CORRECT_CREDENTIALS = {
  userIdentifier: 'user_with_payment_pin',
  password: 'test1234',
};

const INCORRECT_CREDENTIALS = {
  userIdentifier: 'delia.smith',
  password: 'this_is_incorrect',
};

const CAPTCHA_CREDENTIALS = {
  userIdentifier: 'user_with_sign_in_captcha',
  password: 'test1234',
};

const INCORRECT_CAPTCHA_CREDENTIALS = {
  userIdentifier: 'user_with_sign_in_captcha',
  password: 'blugh',
};

const SKY_ID_TOKEN = '01-valid_user';
const GET_PROFILE_URL = '/modifyprofile/display/service/international';
const SIGN_IN_URL = '/signin/service/international';
const SIGN_UP_URL = '/signup/service/international';
const SIGN_IN_CAPTCHA_URL = '/signin/securitycheck/service/international';
const RESET_PASSWORD_REQUEST_URL = '/resetpassword/service/international';
const RESET_PASSWORD_FORM_URL = '/resetpassword/byemail/tokeny-token/service/international';
const RESET_PASSWORD_FORM_URL_EXPIRED = '/resetpassword/byemail/expired-token/service/international';
const RESET_PASSWORD_URL = '/resetpassword/byemail/reset/service/international';

const SIREN_CONTENT_TYPE = 'application/vnd.siren+json';

describe('Sky ID', () => {
  let server;

  beforeAll(() => {
    server = app.listen();
  });

  afterAll(() => {
    server.close();
  });

  describe('Get Profile', () => {
    it('should return 200', () => {
      return supertest(server)
        .get(GET_PROFILE_URL)
        .set(CORRECT_HEADERS)
        .set('Cookie', 'skyCEsidismesso01=01-valid_user')
        .set('Cookie', 'skyCEsidmesso01=01-valid_user')
        .expect(200);
    });

    it('should return 400 if there is no user cookie', () => {
      return supertest(server)
        .get(GET_PROFILE_URL)
        .set(CORRECT_HEADERS)
        .expect(401);
    });
  });

  describe('Sign in', () => {
    describe('GET /signin/service/international', () => {
      it('should return 200', () => {
        return supertest(server)
          .get(SIGN_IN_URL)
          .set(CORRECT_HEADERS)
          .expect(200);
      });

      it('should return 400 if a header is missing', () => {
        return supertest(server)
          .get(SIGN_IN_URL)
          .set(ONE_MISSING_HEADER)
          .expect(400);
      });

      it('should replace http://localhost:5000 with the current origin that the server is hit at', () => {
        return supertest(server)
          .get(SIGN_IN_URL)
          .set(CORRECT_HEADERS)
          .expect(res => {
            const body = JSON.stringify(res.body);
            expect(body).not.toContain('http://localhost:5000');
            expect(body).toContain('http://local.nowtv.com:5000');
          })
          .expect(200);
      });

      it('should not replace http://localhost:5000 with the current origin that the server is hit at if the origin header does not exist', () => {
        return supertest(server)
          .get(SIGN_IN_URL)
          .set(MISSING_ORIGIN_HEADER)
          .expect(res => {
            const body = JSON.stringify(res.body);
            expect(body).toContain('http://localhost:5000');
            expect(body).not.toContain('http://local.nowtv.com:5000');
          })
          .expect(200);
      });
    });

    describe('POST /signin/service/international', () => {
      let req;

      beforeEach(() => {
        req = supertest(server).post(SIGN_IN_URL);
      });

      it('should return 201 and response body if successful', () => {
        const cookies =
          `skyCEsidismesso01=${SKY_ID_TOKEN}; path=/,` +
          `skyCEsidmesso01=${SKY_ID_TOKEN}; path=/,` +
          `skySSO=${SKY_ID_TOKEN}; path=/`;
        return req
          .set(CORRECT_HEADERS)
          .send(CORRECT_CREDENTIALS)
          .then(res => {
            expect(res.headers['set-cookie']).toEqual([cookies]);
            expect(res.status).toEqual(201);
          });
      });

      it('should return captcha response and set preCaptcha cookies for captcha user if credentials correct', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(CAPTCHA_CREDENTIALS)
          .expect(res => {
            expect(res.body.class).toContain('securitycheck');
          })
          .then(res => {
            expect(res.headers['set-cookie']).toEqual([
              'preCaptchaCredentialsCorrect=true; path=/,preCaptchaUserName=user_with_sign_in_captcha; path=/',
            ]);
            expect(res.status).toEqual(200);
          });
      });

      it('should return captcha response and set preCaptcha cookies for captcha user if credentials wrong', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(INCORRECT_CAPTCHA_CREDENTIALS)
          .expect(res => {
            expect(res.body.class).toContain('securitycheck');
          })
          .then(res => {
            expect(res.headers['set-cookie']).toEqual([
              'preCaptchaCredentialsCorrect=false; path=/,preCaptchaUserName=user_with_sign_in_captcha; path=/',
            ]);
            expect(res.status).toEqual(200);
          });
      });

      it('should return a different SSO and response body for different users', () => {
        const cookies =
          `skyCEsidismesso01=${users[OTHER_CORRECT_CREDENTIALS.userIdentifier].skyIdToken}; path=/,` +
          `skyCEsidmesso01=${users[OTHER_CORRECT_CREDENTIALS.userIdentifier].skyIdToken}; path=/,` +
          `skySSO=${users[OTHER_CORRECT_CREDENTIALS.userIdentifier].skyIdToken}; path=/`;
        return req
          .set(CORRECT_HEADERS)
          .send(OTHER_CORRECT_CREDENTIALS)
          .then(res => {
            expect(res.headers['set-cookie']).toEqual([cookies]);
            expect(res.status).toEqual(201);
          });
      });

      it('should return 400 if a header is missing', () => {
        return req
          .set(ONE_MISSING_HEADER)
          .send(CORRECT_CREDENTIALS)
          .expect(400);
      });

      it('should return 422 and correct category error if credentials are incorrect', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(INCORRECT_CREDENTIALS)
          .expect(422, /.*Sorry, we did not recognise either your username or password*/);
      });

      it('should return 500 if Content-Type isnt x-www-form-urlencoded', () => {
        return req
          .set(INCORRECT_CONTENT_TYPE)
          .send('plain text')
          .expect(500);
      });
    });

    describe('POST /signin/securitycheck/service/international', () => {
      let req;

      beforeEach(() => {
        req = supertest(server).post(SIGN_IN_CAPTCHA_URL);
      });

      it('should return 201 if captcha solved and credentials correct', () => {
        return req
          .set(CORRECT_HEADERS)
          .set('Cookie', 'preCaptchaUserName=user_with_sign_in_captcha; preCaptchaCredentialsCorrect=true')
          .send({
            captcha: 'kntmnk',
            captchaId: 'test',
          })
          .expect(res => {
            expect(res.body.class).toContain('success');
          })
          .expect(201);
      });

      it('should return 422 and sign in form with error if captcha solved but credentials wrong', () => {
        return req
          .set(CORRECT_HEADERS)
          .set('Cookie', 'preCaptchaUserName=user_with_sign_in_captcha; preCaptchaCredentialsCorrect=false')
          .send({
            captcha: 'kntmnk',
            captchaId: 'test',
          })
          .expect(res => {
            expect(res.body.class).toContain('signin');
          })
          .expect(422);
      });

      it('should return 422 and captcha error if captcha not solved', () => {
        return req
          .set(CORRECT_HEADERS)
          .set('Cookie', 'preCaptchaUserName=user_with_sign_in_captcha; preCaptchaCredentialsCorrect=true')
          .send({
            captcha: 'bluuugh',
            captchaId: 'test',
          })
          .expect(res => {
            expect(res.body.class).toContain('securitycheck');
          })
          .expect(422);
      });
    });
  });

  describe('Sign up', () => {
    describe('POST /signup/service/international', () => {
      let req;
      let credentials;

      beforeEach(() => {
        req = supertest(server).post(SIGN_UP_URL);
        credentials = Object.assign({ captcha: 'vfchq' }, CORRECT_SIGN_UP_FORM);
      });

      it('should return 200', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(credentials)
          .expect(200);
      });

      it('should return 400 if a header is missing', () => {
        return req
          .set(ONE_MISSING_HEADER)
          .send(credentials)
          .expect(400);
      });

      it('should return 422 if credentials are not provided', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(INCORRECT_INVALID_SIGN_UP_FORM)
          .expect(422);
      });

      it('should return 422 if the form posted does not contain the fields required', () => {
        return req
          .set(CORRECT_HEADERS)
          .send(INCORRECT_EMPTY_SIGN_UP_FORM)
          .expect(422);
      });
    });
  });

  describe('Reset password', () => {
    describe('GET /resetpassword/service/international', () => {
      it('should return 200 if headers correct', () => {
        return supertest(server)
          .get(RESET_PASSWORD_REQUEST_URL)
          .set(BASE_HEADERS)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(200);
      });

      it('should return 400 if a header is missing', () => {
        return supertest(server)
          .get(RESET_PASSWORD_REQUEST_URL)
          .expect(400);
      });
    });

    describe('POST /resetpassword/service/international', () => {
      const correctData = {
        userIdentifier: 'nowtv.test.100@gmail.com',
        resetPasswordUrl: 'https://www.nowtv.com/ie/reset-password',
        _csrf: '4d5454ea-fcba-41ca-a545-2068e2e9a801',
      };

      it('should return 201 if headers and data correct', () => {
        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(CORRECT_HEADERS)
          .send(correctData)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(201);
      });

      it('should return 500 if the wrong content type', () => {
        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(INCORRECT_CONTENT_TYPE)
          .send('potatoes')
          .expect(500);
      });

      it('should return 400 if a header is missing', () => {
        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(ONE_MISSING_HEADER)
          .send(correctData)
          .expect(400);
      });

      it('should return 400 if CSRF value is wrong', () => {
        const data = Object.assign({}, correctData, {
          _csrf: 'goff-woof-goff',
        });

        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(400);
      });

      it('should return 422 if user identifier is missing', () => {
        const data = Object.assign({}, correctData);
        delete data.userIdentifier;

        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(422);
      });

      it('should return 422 if reset password url is missing', () => {
        const data = Object.assign({}, correctData);
        delete data.resetPasswordUrl;

        return supertest(server)
          .post(RESET_PASSWORD_REQUEST_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(422);
      });
    });

    describe('GET /resetpassword/byemail/{token}/service/international', () => {
      it('should return 200 if headers correct', () => {
        return supertest(server)
          .get(RESET_PASSWORD_FORM_URL)
          .set(BASE_HEADERS)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(200);
      });

      it('should return 400 if a header is missing', () => {
        return supertest(server)
          .get(RESET_PASSWORD_FORM_URL)
          .expect(400);
      });

      it('should return 404 if token is expired', () => {
        return supertest(server)
          .get(RESET_PASSWORD_FORM_URL_EXPIRED)
          .set(BASE_HEADERS)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(404);
      });
    });

    describe('POST /resetpassword/byemail/reset/service/international', () => {
      const correctData = {
        password: 'LovingLife2k17',
        confirmPassword: 'LovingLife2k17',
        enterNewPasswordId: '30b45dcb-719d-4835-849e-ce42bd4f3845',
        _csrf: '4d092de9-d44f-4958-af2e-837f612769e9',
      };

      it('should return 201 if headers and data correct', () => {
        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(correctData)
          .expect('Content-Type', SIREN_CONTENT_TYPE)
          .expect(201);
      });

      it('should return 500 if the wrong content type', () => {
        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(INCORRECT_CONTENT_TYPE)
          .send('reset for hackerz')
          .expect(500);
      });

      it('should return 400 if a header is missing', () => {
        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(ONE_MISSING_HEADER)
          .send(correctData)
          .expect(400);
      });

      it('should return 400 if password is missing', () => {
        const data = Object.assign({}, correctData);
        delete data.password;

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(400);
      });

      it('should return 400 if confirmPassword is missing', () => {
        const data = Object.assign({}, correctData);
        delete data.confirmPassword;

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(400);
      });

      it('should return 400 if CSRF value is missing', () => {
        const data = Object.assign({}, correctData, {
          _csrf: undefined,
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(400);
      });

      it('should return 400 if enterNewPasswordId value is missing', () => {
        const data = Object.assign({}, correctData, {
          enterNewPasswordId: undefined,
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(400);
      });

      it('should return 412 if CSRF is expired', () => {
        const data = Object.assign({}, correctData, {
          password: 'csrf_expired1',
          confirmPassword: 'csrf_expired1',
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(412);
      });

      it('should return 422 if password is too short', () => {
        const data = Object.assign({}, correctData, {
          password: 'nop',
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(422);
      });

      it('should return 422 if password is too long', () => {
        const data = Object.assign({}, correctData, {
          password: 'sky-uk/atlas-web-and-tv/packages/atlas-mock-proxy-server',
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(422);
      });

      it('should return 422 if passwords do not match', () => {
        const data = Object.assign({}, correctData, {
          password: 'hello-there',
          confirmPassword: 'goodbye-you',
        });

        return supertest(server)
          .post(RESET_PASSWORD_URL)
          .set(CORRECT_HEADERS)
          .send(data)
          .expect(422);
      });
    });
  });
});
