'use strict';

const supertest = require('supertest');
const sinon = require('sinon');

const app = require('../app');

const REQUEST_BODY = {
  auth: {
    authToken: '01-valid_user',
    authScheme: 'MESSO',
    authIssuer: 'SKY',
    provider: 'NOWTV',
    providerTerritory: 'GB',
    proposition: 'NOWTV',
  },
  device: {
    type: 'COMPUTER',
    platform: 'PC',
  },
};

const REQUEST_BODY_WITHOUT_TOKEN = {
  auth: {
    authScheme: 'MESSO',
    authIssuer: 'SKY',
    provider: 'NOWTV',
    providerTerritory: 'GB',
    proposition: 'NOWTV',
  },
  device: {
    type: 'COMPUTER',
    platform: 'PC',
  },
};

const CORRECT_HEADERS = {
  'X-SkyOTT-Provider': 'NOWTV',
  'Content-Type': 'application/vnd.tokens.v1+json',
  'Content-MD5': '1abe13fc21b2e02a5f73e6c420038c8e',
};

const CORRECT_HEADERS_WITHOUT_TOKEN = {
  'X-SkyOTT-Provider': 'NOWTV',
  'Content-Type': 'application/vnd.tokens.v1+json',
  'Content-MD5': 'c3bc09444792090cb5b548fe7f7e4060',
};

const INVALID_SIGNATURE_HEADER = {
  'X-SkyOTT-Provider': 'NOWTV',
  'Content-Type': 'application/vnd.tokens.v1+json',
  'Content-MD5': 'invalid',
};

describe('User Token Management', () => {
  let clock;
  let server;

  beforeAll(() => {
    clock = sinon.useFakeTimers(1457621019000);
    server = app.listen();
  });

  afterAll(() => {
    clock.restore();
    server.close();
  });

  it('should return 200 and a userToken for the correct user with a valid request', () => {
    return supertest(server)
      .post('/auth/tokens')
      .set(CORRECT_HEADERS)
      .send(REQUEST_BODY)
      .expect({
        userToken: 'valid_user',
      });
  });

  it('should return 403 if the signature is not valid', () => {
    return supertest(server)
      .post('/auth/tokens')
      .set(INVALID_SIGNATURE_HEADER)
      .send(REQUEST_BODY)
      .expect(403, {
        errorCode: 'OVP_00306',
        description: 'Security failure',
      });
  });

  it('returns a 400 if the token is for a different territory', () => {
    const UkUserRequestBody = {
      ...REQUEST_BODY,
      auth: {
        ...REQUEST_BODY.auth,
        authToken: '01-uk_user',
      },
    };

    const ukUserRequestHeaders = {
      ...CORRECT_HEADERS,
      'Content-MD5': '48ba261af425b322ea2ce07f42d1505f',
    };

    return supertest(server)
      .post('/auth/tokens')
      .set(ukUserRequestHeaders)
      .send(UkUserRequestBody)
      .expect(400, {
        errorCode: 'OVP_00026',
        description: 'Required terms and conditions not accepted',
      });
  });

  it('should return 403 if the authToken is not valid', () => {
    return supertest(server)
      .post('/auth/tokens')
      .set(CORRECT_HEADERS_WITHOUT_TOKEN)
      .send(REQUEST_BODY_WITHOUT_TOKEN)
      .expect(403, {
        errorCode: 'OVP_00200',
        description: 'Invalid authentication token',
      });
  });
});
